{% load static i18n %}
<script defer src="{% static 'djadyen/djadyen_advanced.js' %}" type="text/javascript"></script>
<link rel="stylesheet" rel="preload" as="style" href="{% static 'djadyen/djadyen.css' %}"></link>
<div
    id="djadyen-config"
    data-amount="{{ amount | safe }}"
    data-client-key="{{ client_key }}"
    data-country-code="{{ country_code }}"
    data-csrf-token="{{ csrf_token }}"
    data-currency="{{ currency }}"
    data-environment="{{ environment }}"
    data-language="{{ language }}"
    data-payment-type="{% firstof payment_type 'dropin' %}"
    data-payments-endpoint="{{ payments_api }}"
    data-payment-details-endpoint="{{ payment_details_api }}"
    data-redirect-url="{{ redirect_url }}"
    {% if issuer %}data-issuer="{{ issuer }}"{% endif %}
    {% if issuers %}data-issuers="{{ issuers }}"{% endif %}
    {% if adyen_styles_json %}data-styles='{{ adyen_styles_json }}'{% endif %}
></div>
<div id="djadyen-advanced-container"></div>
